package javax.swing.text;

import java.util.Arrays;
import javax.swing.event.DocumentEvent;
import javax.swing.SizeRequirements;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Shape;
import java.awt.font.TextAttribute;

public class ParagraphView extends FlowView implements TabExpander
{
    private int justification;
    private float lineSpacing;
    protected int firstLineIndent;
    private int tabBase;
    static Class i18nStrategy;
    static char[] tabChars;
    static char[] tabDecimalChars;
    
    public ParagraphView(Element element) {
        super(element, 1);
        this.firstLineIndent = 0;
        this.setPropertiesFromAttributes();
        Object property = element.getDocument().getProperty("i18n");
        if (property != null && property.equals(Boolean.TRUE)) {
            try {
                if (ParagraphView.i18nStrategy == null) {
                    String s = "javax.swing.text.TextLayoutStrategy";
                    ClassLoader classLoader = this.getClass().getClassLoader();
                    if (classLoader != null) {
                        ParagraphView.i18nStrategy = classLoader.loadClass(s);
                    }
                    else {
                        ParagraphView.i18nStrategy = Class.forName(s);
                    }
                }
                Object instance = ParagraphView.i18nStrategy.newInstance();
                if ((instance instanceof FlowView.FlowStrategy)) {
                    this.strategy = (FlowView.FlowStrategy)instance;
                }
            }
            catch (Throwable t) {
                throw new StateInvariantError("ParagraphView: Can't create i18n strategy: " + t.getMessage());
            }
        }
    }
    
    protected void setJustification(int justification) {
        this.justification = justification;
    }
    
    protected void setLineSpacing(float lineSpacing) {
        this.lineSpacing = lineSpacing;
    }
    
    protected void setFirstLineIndent(float n) {
        this.firstLineIndent = (int)n;
    }
    
    protected void setPropertiesFromAttributes() {
        AttributeSet attributes = this.getAttributes();
        if (attributes != null) {
            this.setParagraphInsets(attributes);
            Integer n = (Integer)attributes.getAttribute(StyleConstants.Alignment);
            int intValue;
            if (n == null) {
                Object property = this.getElement().getDocument().getProperty(TextAttribute.RUN_DIRECTION);
                if (property != null && property.equals(TextAttribute.RUN_DIRECTION_RTL)) {
                    intValue = 2;
                }
                else {
                    intValue = 0;
                }
            }
            else {
                intValue = (int)n;
            }
            this.setJustification(intValue);
            this.setLineSpacing(StyleConstants.getLineSpacing(attributes));
            this.setFirstLineIndent(StyleConstants.getFirstLineIndent(attributes));
        }
    }
    
    protected int getLayoutViewCount() {
        return this.layoutPool.getViewCount();
    }
    
    protected View getLayoutView(int n) {
        return this.layoutPool.getView(n);
    }
    
    @Override
    protected int getNextNorthSouthVisualPositionFrom(int n, Position.Bias bias, Shape shape, int n2, Position.Bias[] array) throws BadLocationException {
        int n3;
        if (n == -1) {
            n3 = ((n2 == 1) ? (this.getViewCount() - 1) : 0);
        }
        else {
            if (bias == Position.Bias.Backward && n > 0) {
                n3 = this.getViewIndexAtPosition(n - 1);
            }
            else {
                n3 = this.getViewIndexAtPosition(n);
            }
            if (n2 == 1) {
                if (n3 == 0) {
                    return -1;
                }
                --n3;
            }
            else if (++n3 >= this.getViewCount()) {
                return -1;
            }
        }
        JTextComponent textComponent = (JTextComponent)this.getContainer();
        Caret caret = textComponent.getCaret();
        Point point = (caret != null) ? caret.getMagicCaretPosition() : null;
        int n4;
        if (point == null) {
            Rectangle modelToView;
            try {
                modelToView = textComponent.getUI().modelToView(textComponent, n, bias);
            }
            catch (BadLocationException ex) {
                modelToView = null;
            }
            if (modelToView == null) {
                n4 = 0;
            }
            else {
                n4 = modelToView.getBounds().x;
            }
        }
        else {
            n4 = point.x;
        }
        return this.getClosestPositionTo(n, bias, shape, n2, array, n3, n4);
    }
    
    protected int getClosestPositionTo(int n, Position.Bias bias, Shape shape, int n2, Position.Bias[] array, int n3, int n4) throws BadLocationException {
        JTextComponent textComponent = (JTextComponent)this.getContainer();
        Document document = this.getDocument();
        View view = this.getView(n3);
        int i = -1;
        array[0] = Position.Bias.Forward;
        for (int j = 0; j < view.getViewCount(); ++j) {
            View view2 = view.getView(j);
            int startOffset = view2.getStartOffset();
            if (AbstractDocument.isLeftToRight(document, startOffset, startOffset + 1)) {
                i = startOffset;
                int endOffset = view2.getEndOffset();
                while (i < endOffset) {
                    float n6 = (float)textComponent.modelToView(i).getBounds().x;
                    if (n6 >= (float)n4) {
                        while (++i < endOffset && (float)textComponent.modelToView(i).getBounds().x == n6) {}
                        return --i;
                    }
                    ++i;
                }
                --i;
            }
            else {
                i = view2.getEndOffset() - 1;
                while (i >= startOffset) {
                    float n5 = (float)textComponent.modelToView(i).getBounds().x;
                    if (n5 >= (float)n4) {
                        while (--i >= startOffset && (float)textComponent.modelToView(i).getBounds().x == n5) {}
                        return ++i;
                    }
                    --i;
                }
                ++i;
            }
        }
        if (i == -1) {
            return this.getStartOffset();
        }
        return i;
    }
    
    @Override
    protected boolean flipEastAndWestAtEnds(int startOffset, Position.Bias bias) {
        Document document = this.getDocument();
        startOffset = this.getStartOffset();
        return !AbstractDocument.isLeftToRight(document, startOffset, startOffset + 1);
    }
    
    @Override
    public int getFlowSpan(int n) {
        View view = this.getView(n);
        int n2 = 0;
        if ((view instanceof ParagraphView.Row)) {
            ParagraphView.Row row = (ParagraphView.Row)view;
            n2 = row.getLeftInset() + row.getRightInset();
        }
        return (this.layoutSpan == Integer.MAX_VALUE) ? this.layoutSpan : (this.layoutSpan - n2);
    }
    
    @Override
    public int getFlowStart(int n) {
        View view = this.getView(n);
        int leftInset = 0;
        if ((view instanceof ParagraphView.Row)) {
            leftInset = ((ParagraphView.Row)view).getLeftInset();
        }
        return this.tabBase + leftInset;
    }
    
    @Override
    protected View createRow() {
        return new ParagraphView.Row(this.getElement());
    }
    
    @Override
    public float nextTabStop(float n, int n2) {
        if (this.justification != 0) {
            return n + 10.0f;
        }
        n -= (float)this.tabBase;
        TabSet tabSet = this.getTabSet();
        if (tabSet == null) {
            return (float)(this.tabBase + ((int)n / 72 + 1) * 72);
        }
        TabStop tabAfter = tabSet.getTabAfter(n + 0.01f);
        if (tabAfter == null) {
            return (float)this.tabBase + n + 5.0f;
        }
        int alignment = tabAfter.getAlignment();
        int n3;
        switch (alignment) {
            case 5: {
                return (float)this.tabBase + tabAfter.getPosition();
            }
            case 1:
            case 2: {
                n3 = this.findOffsetToCharactersInString(ParagraphView.tabChars, n2 + 1);
                break;
            }
            case 4: {
                n3 = this.findOffsetToCharactersInString(ParagraphView.tabDecimalChars, n2 + 1);
                break;
            }
            default: {
                return (float)this.tabBase + tabAfter.getPosition();
            }
        }
        if (n3 == -1) {
            n3 = this.getEndOffset();
        }
        float partialSize = this.getPartialSize(n2 + 1, n3);
        switch (alignment) {
            case 1:
            case 4: {
                return (float)this.tabBase + Math.max(n, tabAfter.getPosition() - partialSize);
            }
            case 2: {
                return (float)this.tabBase + Math.max(n, tabAfter.getPosition() - partialSize / 2.0f);
            }
            default: {
                return n;
            }
        }
    }
    
    protected TabSet getTabSet() {
        return StyleConstants.getTabSet(this.getElement().getAttributes());
    }
    
    protected float getPartialSize(int n, int a) {
        float n2 = 0.0f;
        this.getViewCount();
        int endOffset;
        for (int elementIndex = this.getElement().getElementIndex(n), viewCount = this.layoutPool.getViewCount(); n < a && elementIndex < viewCount; n = endOffset) {
            View view = this.layoutPool.getView(elementIndex++);
            endOffset = view.getEndOffset();
            int min = Math.min(a, endOffset);
            if ((view instanceof TabableView)) {
                n2 += ((TabableView)view).getPartialSpan(n, min);
            }
            else if (n == view.getStartOffset() && min == view.getEndOffset()) {
                n2 += view.getPreferredSpan(0);
            }
            else {
                return 0.0f;
            }
        }
        return n2;
    }
    
    protected int findOffsetToCharactersInString(char[] array, int n) {
        int length = array.length;
        int endOffset = this.getEndOffset();
        Segment segment = new Segment();
        try {
            this.getDocument().getText(n, endOffset - n, segment);
        }
        catch (BadLocationException ex) {
            return -1;
        }
        for (int i = segment.offset; i < segment.offset + segment.count; ++i) {
            int n2 = segment.array[i];
            int j = 0;
            while (j < length) {
                if (n2 == array[j]) {
                    return i - segment.offset + n;
                }
                ++j;
            }
        }
        return -1;
    }
    
    protected float getTabBase() {
        return (float)this.tabBase;
    }
    
    @Override
    public void paint(Graphics graphics, Shape shape) {
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        this.tabBase = rectangle.x + this.getLeftInset();
        super.paint(graphics, shape);
        if (this.firstLineIndent < 0) {
            Shape childAllocation = this.getChildAllocation(0, shape);
            if (childAllocation != null && childAllocation.intersects(rectangle)) {
                int n = rectangle.x + this.getLeftInset() + this.firstLineIndent;
                int n2 = rectangle.y + this.getTopInset();
                Rectangle clipBounds = graphics.getClipBounds();
                this.tempRect.x = n + this.getOffset(0, 0);
                this.tempRect.y = n2 + this.getOffset(1, 0);
                this.tempRect.width = this.getSpan(0, 0) - this.firstLineIndent;
                this.tempRect.height = this.getSpan(1, 0);
                if (this.tempRect.intersects(clipBounds)) {
                    this.tempRect.x -= this.firstLineIndent;
                    this.paintChild(graphics, this.tempRect, 0);
                }
            }
        }
    }
    
    @Override
    public float getAlignment(int i) {
        switch (i) {
            case 1: {
                float n = 0.5f;
                if (this.getViewCount() != 0) {
                    int n2 = (int)this.getPreferredSpan(1);
                    int n3 = (int)this.getView(0).getPreferredSpan(1);
                    n = ((n2 != 0) ? ((float)(n3 / 2) / (float)n2) : 0.0f);
                }
                return n;
            }
            case 0: {
                return 0.5f;
            }
            default: {
                throw new IllegalArgumentException("Invalid axis: " + i);
            }
        }
    }
    
    public View breakView(int n, float n2, Shape shape) {
        if (n == 1) {
            if (shape != null) {
                Rectangle bounds = shape.getBounds();
                this.setSize((float)bounds.width, (float)bounds.height);
                return this;
            }
            return this;
        }
        else {
            return this;
        }
    }
    
    public int getBreakWeight(int n, float n2) {
        if (n == 1) {
            return 0;
        }
        return 0;
    }
    
    @Override
    protected SizeRequirements calculateMinorAxisRequirements(int n, SizeRequirements calculateMinorAxisRequirements) {
        calculateMinorAxisRequirements = super.calculateMinorAxisRequirements(n, calculateMinorAxisRequirements);
        float n2 = 0.0f;
        float b = 0.0f;
        for (int layoutViewCount = this.getLayoutViewCount(), i = 0; i < layoutViewCount; ++i) {
            View layoutView = this.getLayoutView(i);
            float minimumSpan = layoutView.getMinimumSpan(n);
            if (layoutView.getBreakWeight(n, 0.0f, layoutView.getMaximumSpan(n)) > 0) {
                int startOffset = layoutView.getStartOffset();
                int endOffset = layoutView.getEndOffset();
                float edgeSpan = this.findEdgeSpan(layoutView, n, startOffset, startOffset, endOffset);
                float edgeSpan2 = this.findEdgeSpan(layoutView, n, endOffset, startOffset, endOffset);
                n2 = Math.max(n2, Math.max(minimumSpan, b + edgeSpan));
                b = edgeSpan2;
            }
            else {
                b += minimumSpan;
                n2 = Math.max(n2, b);
            }
        }
        calculateMinorAxisRequirements.minimum = Math.max(calculateMinorAxisRequirements.minimum, (int)n2);
        calculateMinorAxisRequirements.preferred = Math.max(calculateMinorAxisRequirements.minimum, calculateMinorAxisRequirements.preferred);
        calculateMinorAxisRequirements.maximum = Math.max(calculateMinorAxisRequirements.preferred, calculateMinorAxisRequirements.maximum);
        return calculateMinorAxisRequirements;
    }
    
    private float findEdgeSpan(View view, int n, int n2, int n3, int n4) {
        int n5 = n4 - n3;
        if (n5 <= 1) {
            return view.getMinimumSpan(n);
        }
        int n6 = n3 + n5 / 2;
        int n7 = n6 > n2;
        View view2 = n7 ? view.createFragment(n2, n6) : view.createFragment(n6, n2);
        if (((view2.getBreakWeight(n, 0.0f, view2.getMaximumSpan(n)) > 0) ? 1 : 0) == n7) {
            n4 = n6;
        }
        else {
            n3 = n6;
        }
        return this.findEdgeSpan(view2, n, n2, n3, n4);
    }
    
    @Override
    public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.setPropertiesFromAttributes();
        this.layoutChanged(0);
        this.layoutChanged(1);
        super.changedUpdate(documentEvent, shape, viewFactory);
    }
    
    static {
        (ParagraphView.tabChars = new char[1])[0] = '\t';
        (ParagraphView.tabDecimalChars = new char[2])[0] = '\t';
        ParagraphView.tabDecimalChars[1] = '.';
    }
    
    class Row extends BoxView
    {
        static final int SPACE_ADDON = 0;
        static final int SPACE_ADDON_LEFTOVER_END = 1;
        static final int START_JUSTIFIABLE = 2;
        static final int END_JUSTIFIABLE = 3;
        int[] justificationData;
        
        Row(Element element) {
            super(element, 0);
            this.justificationData = null;
        }
        
        @Override
        protected void loadChildren(ViewFactory viewFactory) {
        }
        
        @Override
        public AttributeSet getAttributes() {
            View parent = this.getParent();
            return (parent != null) ? parent.getAttributes() : null;
        }
        
        @Override
        public float getAlignment(int n) {
            if (n == 0) {
                switch (ParagraphView.this.justification) {
                    case 0: {
                        return 0.0f;
                    }
                    case 2: {
                        return 1.0f;
                    }
                    case 1: {
                        return 0.5f;
                    }
                    case 3: {
                        float n2 = 0.5f;
                        if (this.isJustifiableDocument()) {
                            n2 = 0.0f;
                        }
                        return n2;
                    }
                    default: {
                        break;
                    }
                }
            }
            return super.getAlignment(n);
        }
        
        @Override
        public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
            View viewAtPosition = this.getViewAtPosition(n, shape.getBounds());
            if (viewAtPosition != null && !viewAtPosition.getElement().isLeaf()) {
                return super.modelToView(n, shape, bias);
            }
            Rectangle bounds = shape.getBounds();
            int height = bounds.height;
            int y = bounds.y;
            Rectangle bounds2 = super.modelToView(n, shape, bias).getBounds();
            bounds2.height = height;
            bounds2.y = y;
            return bounds2;
        }
        
        @Override
        public int getStartOffset() {
            int min = Integer.MAX_VALUE;
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                min = Math.min(min, this.getView(i).getStartOffset());
            }
            return min;
        }
        
        @Override
        public int getEndOffset() {
            int max = 0;
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                max = Math.max(max, this.getView(i).getEndOffset());
            }
            return max;
        }
        
        @Override
        protected void layoutMinorAxis(int n, int n2, int[] array, int[] array2) {
            this.baselineLayout(n, n2, array, array2);
        }
        
        @Override
        protected SizeRequirements calculateMinorAxisRequirements(int n, SizeRequirements sizeRequirements) {
            return this.baselineRequirements(n, sizeRequirements);
        }
        
        private boolean isLastRow() {
            View parent;
            return (parent = this.getParent()) == null || this == parent.getView(parent.getViewCount() - 1);
        }
        
        private boolean isBrokenRow() {
            boolean n = false;
            int viewCount = this.getViewCount();
            if (viewCount > 0 && this.getView(viewCount - 1).getBreakWeight(0, 0.0f, 0.0f) >= 3000) {
                n = true;
            }
            return n;
        }
        
        private boolean isJustifiableDocument() {
            return !Boolean.TRUE.equals(this.getDocument().getProperty("i18n"));
        }
        
        private boolean isJustifyEnabled() {
            return ParagraphView.this.justification == 3 && this.isJustifiableDocument() && !this.isLastRow() && !this.isBrokenRow();
        }
        
        @Override
        protected SizeRequirements calculateMajorAxisRequirements(int n, SizeRequirements sizeRequirements) {
            int[] justificationData = this.justificationData;
            this.justificationData = null;
            SizeRequirements calculateMajorAxisRequirements = super.calculateMajorAxisRequirements(n, sizeRequirements);
            if (this.isJustifyEnabled()) {
                this.justificationData = justificationData;
                return calculateMajorAxisRequirements;
            }
            return calculateMajorAxisRequirements;
        }
        
        @Override
        protected void layoutMajorAxis(int n, int n2, int[] array, int[] array2) {
            int[] justificationData = this.justificationData;
            this.justificationData = null;
            super.layoutMajorAxis(n, n2, array, array2);
            if (!this.isJustifyEnabled()) {
                return;
            }
            int n3 = 0;
            for (int length = array2.length, i = 0; i < length; ++i) {
                n3 += array2[i];
            }
            if (n3 == n) {
                return;
            }
            int n4 = 0;
            int n5 = -1;
            int n6 = -1;
            int leadingSpaces = 0;
            int startOffset = this.getStartOffset();
            int[] a = new int[this.getEndOffset() - startOffset];
            Arrays.fill(a, 0);
            for (int j = this.getViewCount() - 1; j >= 0; --j) {
                View view = this.getView(j);
                if ((view instanceof GlyphView)) {
                    GlyphView.JustificationInfo justificationInfo = ((GlyphView)view).getJustificationInfo(startOffset);
                    int startOffset2 = view.getStartOffset();
                    int n7 = startOffset2 - startOffset;
                    for (int k = 0; k < justificationInfo.spaceMap.length(); ++k) {
                        if (justificationInfo.spaceMap.get(k)) {
                            a[k + n7] = 1;
                        }
                    }
                    if (n5 > 0) {
                        if (justificationInfo.end >= 0) {
                            n4 += justificationInfo.trailingSpaces;
                        }
                        else {
                            leadingSpaces += justificationInfo.trailingSpaces;
                        }
                    }
                    if (justificationInfo.start >= 0) {
                        n5 = justificationInfo.start + startOffset2;
                        n4 += leadingSpaces;
                    }
                    if (justificationInfo.end >= 0 && n6 < 0) {
                        n6 = justificationInfo.end + startOffset2;
                    }
                    n4 += justificationInfo.contentSpaces;
                    leadingSpaces = justificationInfo.leadingSpaces;
                    if (justificationInfo.hasTab) {
                        break;
                    }
                }
            }
            if (n4 <= 0) {
                return;
            }
            int n8 = n - n3;
            int n9 = (n4 > 0) ? (n8 / n4) : 0;
            int n10 = -1;
            for (int n11 = n5 - startOffset, l = n8 - n9 * n4; l > 0; l -= a[n11], ++n11) {
                n10 = n11;
            }
            if (n9 > 0 || n10 >= 0) {
                (this.justificationData = ((justificationData != null) ? justificationData : new int[4]))[0] = n9;
                this.justificationData[1] = n10;
                this.justificationData[2] = n5 - startOffset;
                this.justificationData[3] = n6 - startOffset;
                super.layoutMajorAxis(n, n2, array, array2);
            }
            return;
        }
        
        @Override
        public float getMaximumSpan(int n) {
            float maximumSpan;
            if (0 == n && this.isJustifyEnabled()) {
                maximumSpan = Float.MAX_VALUE;
            }
            else {
                maximumSpan = super.getMaximumSpan(n);
            }
            return maximumSpan;
        }
        
        @Override
        protected int getViewIndexAtPosition(int n) {
            if (n < this.getStartOffset() || n >= this.getEndOffset()) {
                return -1;
            }
            int i = this.getViewCount() - 1;
            while (i >= 0) {
                View view = this.getView(i);
                if (n >= view.getStartOffset() && n < view.getEndOffset()) {
                    return i;
                }
                --i;
            }
            return -1;
        }
        
        @Override
        protected short getLeftInset() {
            int firstLineIndent = 0;
            View parent;
            if ((parent = this.getParent()) != null && this == parent.getView(0)) {
                firstLineIndent = ParagraphView.this.firstLineIndent;
            }
            return (short)(super.getLeftInset() + firstLineIndent);
        }
        
        @Override
        protected short getBottomInset() {
            return (short)(int)((float)super.getBottomInset() + (float)((this.minorRequest != null) ? this.minorRequest.preferred : 0) * ParagraphView.this.lineSpacing);
        }
    }
}
